@@ -37,3 +37,17 @@ class RandomIrrelevant(RNGMixin, gym.ObservationWrapper):
         )
         observation = np.concatenate([observation, irrelevant])
         return observation.astype(self.observation_space.dtype)
+
+    @staticmethod
+    def wrap_env_function(func: callable, size: int) -> callable:
+        """Wrap base env reward/termination function to ignore added variables.
+
+        Args:
+            func: Callable for reward/termination function
+            size: Number of irrelevant/redundant variables
+        """
+
+        def env_fn(state, action, next_state):
+            return func(state[..., :-size], action, next_state[..., :-size])
+
+        return env_fn
